package br.ufes.inf.lp.ppgicred.es

import java.text.NumberFormat
import java.time.Instant
import java.util.{Date, Locale}
import java.text.SimpleDateFormat

import br.ufes.inf.lp.ppgicred.dominio._

import scala.io.Source

class Entrada (categoriasQualis: List[String], arquivoDocentes: String, arquivoVeiculos: String, arquivoPublicacoes: String, arquivoQualis: String, arquivoRegras: String) {
  private[this] val dateParser = new SimpleDateFormat("dd/MM/yyyy")
  private[this] val nf = NumberFormat.getInstance(new Locale("pt", "BR"))

  def lerArquivos: DadosEntrada = {
    val dE = new DadosEntrada

    leDocentes(dE)
    leVeiculos(dE)
    lePublicacoes(dE)
    leQualis(dE)
    leRegras(dE)

    dE
  }

  private def leDocentes (dE: DadosEntrada): Unit = {
    Source.fromFile(arquivoDocentes).getLines().drop(1).foreach(line => {
      // Quebra a linha em tokens separados por ; e remove leading e trailing spaces
      // aplicando o método trim em cada elemento do array
      val tokens: Array[String] = line.split(";").map(_.trim)

      val codigo = BigInt(tokens(0))
      val nome = tokens(1)
      val dataNascimento = dateParser.parse(tokens(2))
      val dataIngresso = dateParser.parse(tokens(3))
      val coordenador = tokens.length > 4 && tokens(4) == "X"

      dE.adicionaDocente(new Docente(codigo, nome, dataNascimento, dataIngresso, coordenador))
    })
  }

  private def leVeiculos (dE: DadosEntrada): Unit = {
    Source.fromFile(arquivoVeiculos).getLines().drop(1).foreach(line => {
      // Quebra a linha em tokens separados por ; e remove leading e trailing spaces
      // aplicando o método trim em cada elemento do array
      val tokens: Array[String] = line.split(";").map(_.trim)

      val sigla = tokens(0)
      val nome = tokens(1)
      val tipo = tokens(2).charAt(0)
      var issn: String = null
      val fatorImpacto = nf.parse(tokens(3)).doubleValue

      val v: Veiculo = tipo match {
        case 'C' =>
          new Veiculo(sigla, nome, tipo, fatorImpacto, issn)

        case 'P' =>
          if(tokens.length > 4)
            issn = tokens(4)
          new Veiculo(sigla, nome, tipo, fatorImpacto, issn)
      }

      dE.adicionaVeiculo(v)
    })
  }

  private def lePublicacoes (dE: DadosEntrada): Unit = {
    Source.fromFile(arquivoPublicacoes).getLines().drop(1).foreach(line => {
      val tokens = line.split(";").map(_.trim)

      var volume = -1
      val siglaVeiculo = tokens(1)
      val titulo = tokens(2)
      val local = tokens(6)
      val numero = tokens(4).toInt

      if(tokens(5).nonEmpty)
        volume = tokens(5).toInt

      val paginaInicial = tokens(7).toInt
      val paginaFinal = tokens(8).toInt
      val ano = tokens(0).toInt

      // Transforma a lista de código dos autores de string para int
      val codigoAutores = tokens(3).split(",").map(c => BigInt(c.trim))
      var listaDocentesPublicacao: List[Docente] = List[Docente]()

      codigoAutores.foreach(codigo => {
        listaDocentesPublicacao = dE.getDocenteCodigo(codigo) :: listaDocentesPublicacao
      })

      val p: Publicacao = local match {
        case "" =>
          new PublicacaoPeriodico(ano, numero, paginaInicial, paginaFinal, titulo, listaDocentesPublicacao, volume)
        case _ =>
          new PublicacaoConferencia(ano, numero, paginaInicial, paginaFinal, titulo, listaDocentesPublicacao, local)
      }

      val v: Veiculo = dE.getVeiculo(siglaVeiculo)
      v.adicionaPublicacao(p)
      p.setVeiculo(v)

      dE.adicionaPublicacao(p)

      // Quanto um parâmetro é usado apenas uma vez dentro de uma função, é possível
      // substituílo por um wildcard.
      // o foreach espera uma PF (partial function) que recebe como parâmetro um docente
      // e retorna Unit
      listaDocentesPublicacao.foreach(_.adicionaPublicacao(p))
    })
  }

  private def leQualis (dE: DadosEntrada): Unit = {
    Source.fromFile(arquivoQualis).getLines().drop(1).foreach(line => {
      val tokens = line.split(";").map(_.trim)

      val ano = tokens(0).toInt
      val siglaVeiculo = tokens(1)
      val qualificacao = tokens(2)

      val v: Veiculo = dE.getVeiculo(siglaVeiculo)
      val q = new Qualificacao(ano, qualificacao, v)

      v.adicionaQualificacao(q)
    })
  }

  private def leRegras (dE: DadosEntrada): Unit = {
    Source.fromFile(arquivoRegras).getLines().drop(1).foreach(line => {
      val tokens = line.split(";").map(_.trim)

      val qualis = tokens(2)
      val listaQualis = qualis.split(",").map(_.trim)
      val pontos = tokens(3)
      val listaPontos = pontos.split(",").map(_.trim)

      val dataInicio = dateParser.parse(tokens(0))
      val dataFim = dateParser.parse(tokens(1))

      val listaPontuacao: Array[Int] = new Array[Int](categoriasQualis.length)

      var i = 0
      while(i < listaQualis.length) {
        var j = categoriasQualis.indexOf(listaQualis(i))

        while(
          j < (if(i != listaQualis.length - 1) categoriasQualis.indexOf(listaQualis(i + 1)) else categoriasQualis.length)
        ) {
          val x = listaPontos(i).toInt
          listaPontuacao(j) = x

          j += 1
        }

        i+= 1
      }

      val multiplicador = nf.parse(tokens(4)).doubleValue
      val minimoPontos = nf.parse(tokens(6)).doubleValue
      val anos = nf.parse(tokens(5)).intValue

      val r: Regra = new Regra(dataInicio, dataFim, multiplicador, anos, minimoPontos)

      i = 0
      while(i < listaPontuacao.length) {
        r.adicionaPontuacao(categoriasQualis(i), listaPontuacao(i))
        i += 1
      }

      dE.adicionaRegra(r)
    })
  }
}
